/*
 * The Apache Software License, Version 1.1
 *
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "Xerces" and "Apache Software Foundation" must
 *    not be used to endorse or promote products derived from this
 *    software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache",
 *    nor may "Apache" appear in their name, without prior written
 *    permission of the Apache Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation and was
 * originally based on software copyright (c) 1999, International
 * Business Machines, Inc., http://www.apache.org.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 */
package sax;

import util.Arguments;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.io.UnsupportedEncodingException;

/**
 * SAXBench - derived from 'SAXCount' sample in Xerces-J.
 * @author Ryosuke Nanba <rna@horobi.com>
 * @version 1.0
 */
public class SAXBench {

    /** parser name. */
	private static String parserName = "org.apache.xerces.parsers.SAXParser"; 
    /** driver name. */
	private static String driverName = "sax.SAX1Driver";
    private static boolean setValidation    = false; //defaults
    private static boolean setNameSpaces    = true;
	private static int wRepeat = 0; // warmup
	private static int tRepeat = 1; // timing

    //
    // Public static methods
    //
	public static Driver createDriver() throws Exception {
		return (Driver)Class.forName(driverName).newInstance();
	}

    public static void run(String uri) {
        try {
			System.out.println(uri + ":");
			Runtime rt = Runtime.getRuntime();

			System.gc();
			long initialUsedMemory = rt.totalMemory() - rt.freeMemory();

			Driver driver = createDriver();

			// warmup if needed.
            long beforeLoad = System.currentTimeMillis();
			for (int i=0; i<wRepeat; i++) {
				driver.createParser(parserName, setValidation, setNameSpaces);
				long before = (i==0) ? beforeLoad : System.currentTimeMillis();
				driver.parse(uri);
				long after = System.currentTimeMillis();
				System.out.println("warmup: " + (after - before) + " ms");
            }
			// parse.
			long sum = 0;
			for (int i=0; i<tRepeat; i++) {
				driver.createParser(parserName, setValidation, setNameSpaces);
				long before = System.currentTimeMillis();
				driver.parse(uri);
				long after = System.currentTimeMillis();
				System.out.println("parse: " + (after - before) + " ms");
				sum += after - before;
			}
			System.out.println("avg. = " + (sum/tRepeat) + " ms");
			driver = null;
			System.gc();
			System.out.println("mem. = " 
							   + (rt.totalMemory() - rt.freeMemory()
								  - initialUsedMemory)
							   + " bytes");
        } catch (org.xml.sax.SAXParseException spe) {
            spe.printStackTrace(System.err);
        } catch (org.xml.sax.SAXException se) {
            if (se.getException() != null)
                se.getException().printStackTrace(System.err);
            else
                se.printStackTrace(System.err);
        } catch (Exception e) {
            e.printStackTrace(System.err);
        }
    }

    //
    // Main
    //

                            /** Main program entry point. */
    public static void main(String argv[]) {

        Arguments argopt = new Arguments();
        argopt.setUsage( new String[]
                         { "usage: java sax.SAXBench (options) uri ...","",
                             "options:",
                             "  -p name  Specify SAX parser by name.",
                             "  -d name  Specify parser driver by name.",
                             "  -n | -N  Turn on/off namespace [default=on]",
                             "  -v | -V  Turn on/off validation [default=off]",
                             "  -w N     Warmup the parser N times before timing.",
						     "  -r N     Repeat timing N times.",
                             "  -h       This help screen."}  );


        // is there anything to do?
        if (argv.length == 0) {
            argopt.printUsage();
            System.exit(1);
        }

        argopt.parseArgumentTokens(argv, new char[]  { 'p', 'd', 'w', 'r', 'm' } );

        int   c;
        String arg = null;
        while ( ( arg =  argopt.getlistFiles() ) != null ) {
            outer:
            while ( (c =  argopt.getArguments()) != -1 ){
                switch (c) {
                case 'v':
                    setValidation = true;
                    break;
                case 'V':
                    setValidation = false;
                    break;
                case 'N':
                    setNameSpaces = false;
                    break;
                case 'n':
                    setNameSpaces = true;
                    break;
                case 'p':
                    parserName = argopt.getStringParameter();
                    break;
                case 'd':
                    driverName = argopt.getStringParameter();
                    break;
                case '?':
                case 'h':
                case '-':
                    argopt.printUsage();
                    System.exit(1);
                    break;
                case 'w':
					wRepeat = Integer.parseInt(argopt.getStringParameter());
                    break;
                case 'r':
					tRepeat = Integer.parseInt(argopt.getStringParameter());
                    break;
                case -1:
                    break outer;
                default:
                    break;
                }

            }
            run(arg);
            ///
        }
    } // main(String[])

} // class SAXCount
